# Stubs for pyspark._globals (Python 3.7)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

__ALL__: Any

class _NoValueType:
    def __new__(cls): ...
    def __reduce__(self): ...
