from pyspark import SparkConf as SparkConf
from pyspark.context import SparkContext as SparkContext
from pyspark.sql import SQLContext as SQLContext, SparkSession as SparkSession
from typing import Any

spark: Any
sc: Any
sql: Any
sqlContext: Any
sqlCtx = sqlContext
code: Any
