# Stubs for pyspark.mllib.common (Python 3.5)
#

from typing import Type

long: Type[int]
unicode: Type[str]

def callJavaFunc(sc, func, *args): ...
def callMLlibFunc(name, *args): ...

class JavaModelWrapper:
    def __init__(self, java_model) -> None: ...
    def __del__(self): ...
    def call(self, name, *a): ...

def inherit_doc(cls): ...
