from typing import Any, Optional

def heappush(heap: Any, item: Any) -> None: ...
def heappop(heap: Any): ...
def heapreplace(heap: Any, item: Any): ...
def heappushpop(heap: Any, item: Any): ...
def heapify(x: Any) -> None: ...
def merge(iterables: Any, key: Optional[Any] = ..., reverse: bool = ...) -> None: ...
def nsmallest(n: Any, iterable: Any, key: Optional[Any] = ...): ...
def nlargest(n: Any, iterable: Any, key: Optional[Any] = ...): ...
