# Stubs for pyspark.ml (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from pyspark.ml.base import (
    Estimator as Estimator,
    Model as Model,
    Transformer as Transformer,
    UnaryTransformer as UnaryTransformer,
)
from pyspark.ml.pipeline import Pipeline as Pipeline, PipelineModel as PipelineModel
