from collections import namedtuple
from typing import Any

CallSite = namedtuple("CallSite", "function file linenum")

def first_spark_call(): ...

class SCCallSiteSync:
    def __init__(self, sc: Any) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(self, type: Any, value: Any, tb: Any) -> None: ...
