from pyspark.sql.pandas.utils import (
    require_minimum_pandas_version as require_minimum_pandas_version,
)
from typing import Any, Optional

def infer_eval_type(sig: Any): ...
def check_tuple_annotation(
    annotation: Any, parameter_check_func: Optional[Any] = ...
): ...
def check_iterator_annotation(
    annotation: Any, parameter_check_func: Optional[Any] = ...
): ...
def check_union_annotation(
    annotation: Any, parameter_check_func: Optional[Any] = ...
): ...
