from pyspark.serializers import UTF8Deserializer as UTF8Deserializer, read_int as read_int, write_with_length as write_with_length  # type: ignore[attr-defined]
from typing import Any, Optional

xrange = range

def launch_gateway(conf: Optional[Any] = ..., popen_kwargs: Optional[Any] = ...): ...
def local_connect_and_auth(port: Any, auth_secret: Any): ...
def ensure_callback_server_started(gw: Any) -> None: ...
