# Stubs for pyspark.streaming.util (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

class TransformFunction:
    ctx: Any
    func: Any
    deserializers: Any
    rdd_wrap_func: Any
    failure: Any
    def __init__(self, ctx, func, *deserializers) -> None: ...
    def rdd_wrapper(self, func): ...
    def call(self, milliseconds, jrdds): ...
    def getLastFailure(self): ...
    class Java:
        implements: Any

class TransformFunctionSerializer:
    ctx: Any
    serializer: Any
    gateway: Any
    failure: Any
    def __init__(self, ctx, serializer, gateway: Optional[Any] = ...) -> None: ...
    def dumps(self, id): ...
    def loads(self, data): ...
    def getLastFailure(self): ...
    class Java:
        implements: Any

def rddToFileName(prefix, suffix, timestamp): ...
