from pyspark.serializers import UTF8Deserializer as UTF8Deserializer, read_int as read_int, write_int as write_int, write_with_length as write_with_length  # type: ignore[attr-defined]
from typing import Any

def compute_real_exit_code(exit_code: Any): ...
def worker(sock: Any, authenticated: Any): ...
def manager() -> None: ...
