# Stubs for pyspark.mllib (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# Names in __all__ with no definition:
#   classification
#   clustering
#   feature
#   fpm
#   linalg
#   random
#   recommendation
#   regression
#   stat
#   tree
#   util
