# Stubs for pyspark.streaming (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from pyspark.streaming.context import StreamingContext as StreamingContext
from pyspark.streaming.dstream import DStream as DStream
from pyspark.streaming.listener import StreamingListener as StreamingListener
